//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _PHYSMEM_
#define _PHYSMEM_

#ifndef _UNIIO_H
   #include "uniio.h"
#endif


//-----------------------------------------------------------------------------
namespace Uniio
{

class _Export_UNIIO PhysicalMemory
{
public:

   PhysicalMemory ( const unsigned long inPhysicalAddress
                  , const unsigned long inSize
                  , const bool fWriteAccess = true
                  );

   ~PhysicalMemory ();

   unsigned long size () const
      { return theSize; }

   PhysicalMemory & writeMemoryBlock ( const void * const inData
                                     , const unsigned long inBlockSize
                                     , const unsigned long inStartOffset = 0
                                     );

   const PhysicalMemory & readMemoryBlock ( void * inBuffer
                                    , const unsigned long inBlockSize
                                    , const unsigned long inStartOffset = 0
                                    ) const;


   // these functions provided for convienience
   
   long  readLong  (const unsigned long inStartOffset) const;
   short readShort (const unsigned long inStartOffset) const;
   char  readChar  (const unsigned long inStartOffset) const;

   PhysicalMemory & writeLong  ( const unsigned long inStartOffset
                               , const long inData
                               );
   PhysicalMemory & writeShort ( const unsigned long inStartOffset
                               , const short inData
                               );
   PhysicalMemory & writeChar  ( const unsigned long inStartOffset
                               , const char inData
                               );

protected:

private:

   // disable copy constructor and assignment operator
   PhysicalMemory (const PhysicalMemory & );
   PhysicalMemory & operator = (const PhysicalMemory &);

   void * thePtr;
   const unsigned long theSize;
   const bool theWriteAccess;

}; // end PhysicalMemory class declaration

}; // end namespace

#endif
