//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _BTRACE_
   #include "btrace.hpp"
#endif

#ifndef _OSIFACE_
   #include "osiface.hpp"
#endif

#ifndef _STDEXCEPT_
   #include <stdexcept>
#endif

#ifndef _WINDOWS_
   #define WIN32_LEAN_AND_MEAN
   #include <windows.h>
#endif
#include <winioctl.h>

typedef LARGE_INTEGER PHYSICAL_ADDRESS;
extern "C"
{
   #include "uniiodrv.h"
}

using namespace std;
using namespace Uniio;


OSInterface * OSInterface::theInterface = NULL;

//-----------------------------------------------------------------------------
OSInterface & OSInterface::osinterface ()
{
BFUNCTRACE_DEVELOP ();

//If it is first call to the function - create Operating system interface object
if (!theInterface)
{
   theInterface = new OSInterface;
} // endif

return * theInterface;
} // end OSInterface::osinterface()


//-----------------------------------------------------------------------------
OSInterface::OSInterface ()
{
BFUNCTRACE_DEVELOP ();


// uniio.sys driver may not be loaded. First try to load it.

SC_HANDLE schSCManager
   = OpenSCManager ( NULL
                   , NULL
                   , GENERIC_READ
                   );
if (!schSCManager)
{
   throw runtime_error ("OS Interface: failed to open service manager.");
} // endif

SC_HANDLE schService = OpenService ( schSCManager
                                   , "uniio"
                                   , SERVICE_START
                                   );

if ( schService )
{
   StartService ( schService
                , 0
                , NULL
                );
   CloseServiceHandle ( schService );
} 
else
{
   CloseServiceHandle ( schSCManager );
   throw runtime_error ("OS Interface: uniio.sys driver possibly not installed");
} // endif

CloseServiceHandle ( schSCManager );


// uniio.sys driver should be loaded now. open it to enable direct port io.

theDriverHandle = CreateFile ( "\\\\.\\uniio"
                             , GENERIC_READ | GENERIC_WRITE
                             , 0
                             , NULL
                             , OPEN_EXISTING
                             , FILE_ATTRIBUTE_NORMAL
                             , NULL
                             );
if ( theDriverHandle == INVALID_HANDLE_VALUE)
{
   throw runtime_error ("OS Interface: can not open uniio.sys driver.");
} // endif

} // end constructor


//-----------------------------------------------------------------------------
OSInterface::~OSInterface ()
{
BFUNCTRACE_DEVELOP ();

CloseHandle ( theDriverHandle );
} // end destructor


//-----------------------------------------------------------------------------
void * OSInterface::mapMemory ( unsigned long inAddress
                              , unsigned long inBlockSize
                              , bool fWriteAccess
                              )
{
BFUNCTRACE_DEVELOP ();

PHYSICAL_MEMORY_INFO pmi;
pmi.theBusAddress.HighPart = 0;
pmi.theBusAddress.LowPart = inAddress;
pmi.theLength = inBlockSize;

PVOID virtualAddress;
DWORD cbReturned;
if (!DeviceIoControl ( theDriverHandle
                     , (DWORD) IOCTL_UNIIO_MAP_PHYSICAL_MEMORY
                     , &pmi
                     , sizeof(PHYSICAL_MEMORY_INFO)
                     , &virtualAddress
                     , sizeof(PVOID)
                     , &cbReturned
                     , 0
                     )
   )
{
   throw runtime_error ("OS Interface: can not get pointer to physical memory block.");
} // endif

return virtualAddress;
} // end OSInterface::mapMemory (..)


//-----------------------------------------------------------------------------
OSInterface & OSInterface::unmapMemory ( void * inVirtualAddress )
{
BFUNCTRACE_DEVELOP ();

if (!inVirtualAddress)
{
   return * this;
} // endif

DWORD cbReturned;
if (!DeviceIoControl ( theDriverHandle
                     , (DWORD) IOCTL_UNIIO_UNMAP_PHYSICAL_MEMORY
                     , &inVirtualAddress
                     , sizeof(PVOID)
                     , NULL
                     , 0
                     , &cbReturned
                     , 0
                     )
   )
{
   throw runtime_error ("OS Interface: can not release physical memory block.");
} // endif

return * this;
} // end OSInterface::unmapMemory (..)


//-----------------------------------------------------------------------------
void OSInterface::sleep ( const unsigned long inMicroseconds )
{
BFUNCTRACE_DEVELOP ();

Sleep ( inMicroseconds/1000 );

} // end OSInterface::sleep (..)
